import java_cup.runtime.*;
import java.io.*;

public class scanner implements Scanner {
    /* single lookahead character */
    protected static int next_char;

    final InputStream is;

    scanner(String filename) throws java.io.FileNotFoundException {
        is = new FileInputStream(filename);
    }

    /* advance input by one character */
    protected void advance() throws java.io.IOException
    { next_char = is.read(); }

    /* initialize the scanner */
    public void init() throws java.io.IOException
    { advance(); }

    /* recognize and return the next complete token */
    public Symbol next_token() throws java.io.IOException
    {
        for (;;)
            switch (next_char)
                {
                case '0': case '1': case '2': case '3': case '4':
                case '5': case '6': case '7': case '8': case '9':
                    /* parse a float */
                    double x = 0.;
                    do {
                        x = x * 10. + (next_char - '0');
                        advance();
                    } while (next_char >= '0' && next_char <= '9');
                    if (next_char == '.') {
                        double d = 0.1;
                        do {
                            x += d * (next_char - '0');
                            d /= 10.;
                        } while (next_char >= '0' && next_char <= '9');
                    }
                    return new Symbol(sym.FLOAT, x);

                case ';': advance(); return new Symbol(sym.SEQ);
                case '+': advance(); return new Symbol(sym.PLUS);
                case '-': advance(); return new Symbol(sym.MINUS);
                case '*': advance(); return new Symbol(sym.TIMES);
                case '/': advance(); return new Symbol(sym.DIVIDE);
                case '(': advance(); return new Symbol(sym.LPAR);
                case ')': advance(); return new Symbol(sym.RPAR);
                case '{': advance(); return new Symbol(sym.LPAR);
                case '}': advance(); return new Symbol(sym.RPAR);
                case '=': advance(); return new Symbol(sym.EQ);
                case ',': advance(); return new Symbol(sym.COMMA);

                case -1: return new Symbol(sym.EOF);

                default:
                    if (('a' <= next_char && next_char <= 'z') || ('A' <= next_char && next_char <= 'Z')) {
                        String s = "";
                        do {
                            s += Character.toString((char)next_char);
                            advance();
                        } while (('a' <= next_char && next_char <= 'z') || ('A' <= next_char && next_char <= 'Z'));
                        if (s.equals("if"))
                            return new Symbol(sym.IF);
                        else if (s.equals("then"))
                            return new Symbol(sym.THEN);
                        else if (s.equals("else"))
                            return new Symbol(sym.ELSE);
                        else if (s.equals("rand"))
                            return new Symbol(sym.RAND);
                        else
                            return new Symbol(sym.STRING, s);
                    }
                    else
                        /* in this simple scanner we just ignore everything else */
                        advance();
                    break;
                }
    }
};
